Index: drivers/dahdi/wcb4xxp/wcb4xxp.h
===================================================================
--- drivers/dahdi/wcb4xxp/wcb4xxp.h	(révision 5366)
+++ drivers/dahdi/wcb4xxp/wcb4xxp.h	(copie de travail)
@@ -450,6 +450,7 @@
 	struct b4xxp_span spans[MAX_SPANS_PER_CARD];	/* Individual spans */
 	int order;				/* Order */
 	int flags;				/* Device flags */
+	int has_ec;				/* Has Echo Canceller? */
 	int master;				/* Are we master */
 	int ledreg;				/* copy of the LED Register */
 	unsigned int gpio;
Index: drivers/dahdi/wcb4xxp/base.c
===================================================================
--- drivers/dahdi/wcb4xxp/base.c	(révision 5366)
+++ drivers/dahdi/wcb4xxp/base.c	(copie de travail)
@@ -108,9 +108,13 @@
 struct devtype {
 	char *desc;
 	unsigned int flags;
+	int ports; /* FIXME: Assumed to be 4 */
+	int has_ec;
 };
 
-static struct devtype wcb4xxp = { "Wildcard B410P", 0 };
+static struct devtype wcb4xxp = { "Wildcard B410P", .ports = 4, .has_ec = 1 };
+static struct devtype hfc4s =   { "HFC-4S Junghanns.NET quadBRI PCI", .ports = 4 };
+static struct devtype hfc2s =   { "HFC-4S Junghanns.NET duoBRI PCI", .ports = 2 };
 
 
 #if 0
@@ -603,6 +607,9 @@
 	unsigned char b;
 	unsigned int i, j, mask;
 
+	if (! b4->has_ec)
+		return;
+
 /* Setup GPIO */
 	for (i=0; i < NUM_EC; i++) {
 		b = ec_read(b4, i, 0x1a0);
@@ -858,7 +865,8 @@
 
 	if (first) {
 		first = 0;
-		dev_info(b4->dev, "Hardware echo cancellation enabled.\n");
+		if (b4->has_ec)
+			dev_info(b4->dev, "Hardware echo cancellation enabled.\n");
 	}
 
 	fifo = port * 2;
@@ -1389,7 +1397,7 @@
 
 	gpio = b4xxp_getreg8(b4, R_GPI_IN3);
 
-	for (i=0; i < 4; i++) {
+	for (i=0; i < b4->numspans; i++) {
 		s = &b4->spans[i];
 		s->parent = b4;
 		s->port = i;
@@ -1720,7 +1728,7 @@
  * set up the clock controller
  * we have a 24.576MHz crystal, so the PCM clock is 2x the incoming clock.
  */
-	b4xxp_setreg8(b4, R_BRG_PCM_CFG, 0x02);
+	b4xxp_setreg8(b4, R_BRG_PCM_CFG, 0x20);
 	flush_pci();
 
 	udelay(100);				/* wait a bit for clock to settle */
@@ -1815,6 +1823,10 @@
 	int i;
 	struct b4xxp_span *bspan;
 
+	/* FIXME: abusing has_ec as a marker for leds handling? */
+	if (!b4->has_ec)
+		return;
+
 	b4->blinktimer++;
 	for (i=0; i < b4->numspans; i++) {
 		bspan = &b4->spans[i];
@@ -2128,7 +2140,7 @@
 		bspan->span.close  = b4xxp_close;
 		bspan->span.ioctl = b4xxp_ioctl;
 		bspan->span.hdlc_hard_xmit = b4xxp_hdlc_hard_xmit;
-		if (vpmsupport)
+		if (vpmsupport && b4->has_ec)
 			bspan->span.echocan = b4xxp_echocan;
 
 /* HDLC stuff */
@@ -2447,6 +2459,7 @@
 /* card found, enabled and main struct allocated.  Fill it out. */
 	b4->magic = WCB4XXP_MAGIC;
 	b4->variety = dt->desc;
+	b4->has_ec = dt->has_ec;
 
 	b4->pdev = pdev;
 	b4->dev = &pdev->dev;
@@ -2476,7 +2489,8 @@
 */
 
 /* TODO: determine whether this is a 2, 4 or 8 port card */
-	b4->numspans = 4;
+	b4->numspans = dt->ports;
+   vpmsupport = dt->has_ec;
 	b4->syncspan = -1;		/* sync span is unknown */
 	if (b4->numspans > MAX_SPANS_PER_CARD) {
 		dev_err(b4->dev, "Driver does not know how to handle a %d span card!\n", b4->numspans);
@@ -2624,6 +2638,11 @@
 static struct pci_device_id b4xx_ids[] __devinitdata =
 {
 	{ 0xd161, 0xb410, PCI_ANY_ID, PCI_ANY_ID, 0, 0, (unsigned long)&wcb4xxp },
+	/* TODO: this is too generic. Replace with specific entries for 
+	 * specific devices using sub vendor IDs:
+    */
+	{ 0x1397, 0x08b4, 0x1397, 0xb520, 0, 0, (unsigned long)&hfc4s },
+	{ 0x1397, 0x08b4, 0x1397, 0xb556, 0, 0, (unsigned long)&hfc2s },
 	{ 0, }
 };
 
